import {describe, expect, test} from '@jest/globals'
import {BACK_ORDER_ID, BACK_PAY_ID, PayChannel, PayError, PayErrorMessage, Payment, PayType} from "@/Payment/Payment";

describe('测试： 创建实例', () => {
    test('supportCombination', () => {
        const payment1 = new Payment({
            supportCombination: false
        })
        expect(payment1.supportCombination).toBe(false)

        const payment2 = new Payment({
            supportCombination: true
        })
        expect(payment2.supportCombination).toBe(true)

        const payment3 = new Payment()
        expect(payment3.supportCombination).toBe(true)
    })

    test('payChannels', () => {
        const payment = new Payment({
            payChannels: [PayChannel.WX_MWEB, PayChannel.WX_JSAPI]
        })
        expect(payment.limitPayChannels).toEqual([PayChannel.WX_MWEB, PayChannel.WX_JSAPI])
    })

    test('no params', () => {
        const payment = new Payment()
        expect(payment.limitPayChannels).toEqual(Object.keys(PayChannel).map(key => key))
    })

    test('empty array', () => {
        const payment = new Payment({
            payChannels: []
        })
        expect(payment.limitPayChannels).toEqual([])
    })
})

test('测试订单查询链接', () => {
    const testPayId = 111
    const testOrderId = 222
    const backUrl = Payment.createBackInfoUrl(window.location.href, testPayId, testOrderId)
    expect(backUrl).toContain(BACK_PAY_ID)
    expect(backUrl).toContain(BACK_ORDER_ID)
    expect(backUrl).toContain(testPayId.toString())
    expect(backUrl).toContain(testOrderId.toString())
    expect(backUrl).toContain(window.location.origin)
})

test('测试组合支付价格计算', () => {
    const payment = new Payment()
    const goodsPrice = 0.3
    const balance = 0.1
    const {payAmount, payBalance} = payment.computeAmount(goodsPrice, balance)
    expect(payAmount).toBe(0.2)
    expect(payBalance).toBe(0.1)
})

test('测试非组合支付价格计算', () => {
    const payment = new Payment({supportCombination: false})
    const goodsPrice = 0.3
    payment.balance = 0.1
    const {payAmount, payBalance} = payment.computeAmount(goodsPrice)
    expect(payAmount).toBe(0.3)
    expect(payBalance).toBe(0)
})

describe('测试支付入参校验', () => {
    const othersParams = {
        orderId: 111111111111,
        totalAmount: 100,
    }
    test('ali payType', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.WX_JSAPI, PayChannel.WX_JSAPI]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.ALIPAY
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.ALIPAY_PAY_CHANNEL_MISSING)
    })
    test('wechat patType', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.ALI_WAP]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.WECHAT
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.WECHAT_PAY_CHANNEL_MISSING)
    })

    test('wechat redirect missing', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.WX_MWEB, PayChannel.WX_JSAPI]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.WECHAT
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.WECHAT_REDIRECT_URL_MISSING)
    })

    test('wechat redirect encode', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.WX_MWEB, PayChannel.WX_JSAPI]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.WECHAT,
            redirectUrl: 'https://m.ydl.com'
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.WECHAT_REDIRECT_URL_ENCODE)
    })

    test('alipay quitUrl missing', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.ALI_WAP]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.ALIPAY
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.ALIPAY_QUIT_URL_MISSING)
    })

    test('alipay returnUrl missing', async () => {
        const payment = new Payment({
            payChannels: [PayChannel.ALI_WAP]
        })
        const {errorMessage, errorType, success} = await payment.toPay({
            ...othersParams,
            payType: PayType.ALIPAY,
            quitUrl: 'https://m.ydl.com'
        })
        expect(success).toBe(false)
        expect(errorType).toBe(PayError.VALIDATE)
        expect(errorMessage).toBe(PayErrorMessage.ALIPAY_RETURN_URL_MISSING)
    })
})

test('测试微信浏览器支付方式', () => {
    const payment = new Payment()
    const methods = payment.getPayMethodList(100)
    expect(methods.length).toBe(1)
    expect(methods[0].value).toBe(PayType.WECHAT)
})
